/* Copyright (c) 2010 Nordic Semiconductor. All Rights Reserved.
*
* The information contained herein is property of Nordic Semiconductor ASA.
* Terms and conditions of usage are described in detail in NORDIC
* SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
*
* Licensees are granted free, non-transferable use of the information. NO
* WARRANTY of ANY KIND is provided. This heading must NOT be removed from
* the file.
*/

/**
* This file is autogenerated by 1.12.2.2126  
*/

#ifndef SETUP_MESSAGES_H__
#define SETUP_MESSAGES_H__

#include "hal_platform.h" 
#include "aci.h"
#define PIPE_BLESHIELD_VENDOR_NAME_SET 1
#define PIPE_BLESHIELD_RX_DATA_TX 2
#define PIPE_BLESHIELD_TX_DATA_RX 3
#define PIPE_BLESHIELD_RESETWRITE_RX 4
#define PIPE_BLESHIELD_LIB_VER_SET 5
#define PIPE_BLESHIELD_AUTH_TX 6
#define PIPE_BLESHIELD_AUTH_RX 7

#define NUMBER_OF_PIPES 7
typedef struct
{
  aci_pipe_store_t  location;
  aci_pipe_type_t   pipe_type;
} services_pipe_type_mapping_t;

static services_pipe_type_mapping_t services_pipe_type_mapping[NUMBER_OF_PIPES] =
{
  {ACI_STORE_LOCAL, ACI_SET},
  {ACI_STORE_LOCAL, ACI_TX},
  {ACI_STORE_LOCAL, ACI_RX},
  {ACI_STORE_LOCAL, ACI_RX},
  {ACI_STORE_LOCAL, ACI_SET},
  {ACI_STORE_LOCAL, ACI_TX},
  {ACI_STORE_LOCAL, ACI_RX},
};

#define GAP_PPCP_MAX_CONN_INT 0x6 /**< Maximum connection interval as a multiple of 1.25 msec , 0xFFFF means no specific value requested */
#define GAP_PPCP_MIN_CONN_INT  0x6 /**< Minimum connection interval as a multiple of 1.25 msec , 0xFFFF means no specific maximum*/
#define GAP_PPCP_SLAVE_LATENCY 0
#define GAP_PPCP_CONN_TIMEOUT 0x258 /** Connection Supervision timeout multiplier as a multiple of 10msec, 0xFFFF means no specific value requested */

/** @brief do a set_local_data a new value for PIPE_BLESHIELD_VENDOR_NAME_SET
 *  @param src source buffer to send data from
 *  @param size the number of bytes to send. Maximum size is 20
 *  @details use this function to do a set_local_data for PIPE_BLESHIELD_VENDOR_NAME_SET. If no transaction are currently
 *  running, the set will be immediate, otherwise, it will be done at the end of the current transaction
 *  when services_update_pipes will be called.
 */
void services_set_bleshield_vendor_name(void *src, int size);

/** @brief send a new value for PIPE_BLESHIELD_RX_DATA_TX
 *  @param src source buffer to send data from
 *  @param size the number of bytes to send. Maximum size is 20
 *  @param is_freshest_sample set it to true if you want to overwrite an eventual pending transaction on this pipe.
 *  @details use this function to send a new value for PIPE_BLESHIELD_RX_DATA_TX. If no transaction are currently
 *  running, the send will be immediate, otherwise, it will be done at the end of the current transaction
 *  when services_update_pipes will be called. If a transaction on this pipe is already pending, then this function
 *  will not overwrite the data of the previous transaction and return false.
 *  @return : true if is_freshest_sample true, otherwise return false if a transaction on this pipe is already pending, true otherwise.
 */
bool services_send_bleshield_rx_data(void *src, int size, bool is_freshest_sample);

/** @brief do a set_local_data a new value for PIPE_BLESHIELD_LIB_VER_SET
 *  @param src source buffer to send data from
 *  @param size the number of bytes to send. Maximum size is 2
 *  @details use this function to do a set_local_data for PIPE_BLESHIELD_LIB_VER_SET. If no transaction are currently
 *  running, the set will be immediate, otherwise, it will be done at the end of the current transaction
 *  when services_update_pipes will be called.
 */
void services_set_bleshield_lib_ver(void *src, int size);

/** @brief send a new value for PIPE_BLESHIELD_AUTH_TX
 *  @param src source buffer to send data from
 *  @param size the number of bytes to send. Maximum size is 4
 *  @param is_freshest_sample set it to true if you want to overwrite an eventual pending transaction on this pipe.
 *  @details use this function to send a new value for PIPE_BLESHIELD_AUTH_TX. If no transaction are currently
 *  running, the send will be immediate, otherwise, it will be done at the end of the current transaction
 *  when services_update_pipes will be called. If a transaction on this pipe is already pending, then this function
 *  will not overwrite the data of the previous transaction and return false.
 *  @return : true if is_freshest_sample true, otherwise return false if a transaction on this pipe is already pending, true otherwise.
 */
bool services_send_bleshield_auth(void *src, int size, bool is_freshest_sample);

/** @brief function to trig pending transaction on pipes
 *  @details This function check for each pipe if it has a pending transaction (send/rx_request/ack)
 *   and if so executes this transaction.
 *   This function should be called in the APP_RUN state of the process function of the application.
 */
void services_update_pipes(void);

#define NB_SETUP_MESSAGES 28
#define SETUP_MESSAGES_CONTENT {\
    {0x00,\
        {\
            0x07,0x06,0x00,0x00,0x03,0x02,0x40,0x6c,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x07,0x01,0x01,0x00,0x00,0x06,0x00,0x01,\
            0xd0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x10,0x1c,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,\
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x03,0x90,0x01,0x64,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x10,0x38,0x02,0xff,0x02,0x58,0x00,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x00,0x00,\
            0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x05,0x06,0x10,0x54,0x00,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x20,0x00,0x04,0x04,0x02,0x02,0x00,0x01,0x28,0x00,0x01,0x00,0x18,0x04,0x04,0x05,0x05,0x00,\
            0x02,0x28,0x03,0x01,0x0e,0x03,0x00,0x00,0x2a,0x04,0x14,0x10,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x20,0x1c,0x0a,0x00,0x03,0x2a,0x00,0x01,0x42,0x4c,0x45,0x20,0x53,0x68,0x69,0x65,0x6c,0x64,\
            0x2e,0x63,0x6f,0x6d,0x00,0x00,0x04,0x04,0x05,0x05,0x00,0x04,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x20,0x38,0x28,0x03,0x01,0x02,0x05,0x00,0x01,0x2a,0x06,0x04,0x03,0x02,0x00,0x05,0x2a,0x01,\
            0x01,0x0e,0x0d,0x04,0x04,0x05,0x05,0x00,0x06,0x28,0x03,0x01,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x20,0x54,0x02,0x07,0x00,0x04,0x2a,0x06,0x04,0x09,0x08,0x00,0x07,0x2a,0x04,0x01,0x06,0x00,\
            0x06,0x00,0x00,0x00,0x58,0x02,0x04,0x04,0x02,0x02,0x00,0x08,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x20,0x70,0x28,0x00,0x01,0x01,0x18,0x04,0x04,0x10,0x10,0x00,0x09,0x28,0x00,0x01,0x1e,0x94,\
            0x8d,0xf1,0x48,0x31,0x94,0xba,0x75,0x4c,0x3e,0x50,0x00,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x20,0x8c,0x3d,0x71,0x04,0x04,0x13,0x13,0x00,0x0a,0x28,0x03,0x01,0x02,0x0b,0x00,0x1e,0x94,\
            0x8d,0xf1,0x48,0x31,0x94,0xba,0x75,0x4c,0x3e,0x50,0x01,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x20,0xa8,0x3d,0x71,0x04,0x04,0x14,0x00,0x00,0x0b,0x00,0x01,0x02,0x00,0x00,0x00,0x00,0x00,\
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x20,0xc4,0x00,0x00,0x00,0x04,0x04,0x13,0x13,0x00,0x0c,0x28,0x03,0x01,0x10,0x0d,0x00,0x1e,\
            0x94,0x8d,0xf1,0x48,0x31,0x94,0xba,0x75,0x4c,0x3e,0x50,0x02,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x20,0xe0,0x00,0x3d,0x71,0x14,0x00,0x14,0x00,0x00,0x0d,0x00,0x02,0x02,0x00,0x00,0x00,0x00,\
            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x20,0xfc,0x00,0x00,0x00,0x00,0x46,0x14,0x03,0x02,0x00,0x0e,0x29,0x02,0x01,0x00,0x00,0x04,\
            0x04,0x13,0x13,0x00,0x0f,0x28,0x03,0x01,0x04,0x10,0x00,0x1e,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x21,0x18,0x94,0x8d,0xf1,0x48,0x31,0x94,0xba,0x75,0x4c,0x3e,0x50,0x03,0x00,0x3d,0x71,0x44,\
            0x10,0x14,0x00,0x00,0x10,0x00,0x03,0x02,0x00,0x00,0x00,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x21,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,\
            0x04,0x04,0x13,0x13,0x00,0x11,0x28,0x03,0x01,0x04,0x12,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x21,0x50,0x1e,0x94,0x8d,0xf1,0x48,0x31,0x94,0xba,0x75,0x4c,0x3e,0x50,0x04,0x00,0x3d,0x71,\
            0x46,0x10,0x02,0x01,0x00,0x12,0x00,0x04,0x02,0x00,0x04,0x04,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x21,0x6c,0x13,0x13,0x00,0x13,0x28,0x03,0x01,0x02,0x14,0x00,0x1e,0x94,0x8d,0xf1,0x48,0x31,\
            0x94,0xba,0x75,0x4c,0x3e,0x50,0x05,0x00,0x3d,0x71,0x06,0x04,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x21,0x88,0x03,0x02,0x00,0x14,0x00,0x05,0x02,0x02,0x01,0x04,0x04,0x13,0x13,0x00,0x15,0x28,\
            0x03,0x01,0x16,0x16,0x00,0x1e,0x94,0x8d,0xf1,0x48,0x31,0x94,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x21,0xa4,0xba,0x75,0x4c,0x3e,0x50,0x06,0x00,0x3d,0x71,0x56,0x14,0x05,0x04,0x00,0x16,0x00,\
            0x06,0x02,0x00,0x00,0x00,0x00,0x46,0x14,0x03,0x02,0x00,0x17,\
        },\
    },\
    {0x00,\
        {\
            0x09,0x06,0x21,0xc0,0x29,0x02,0x01,0x00,0x00,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x40,0x00,0x00,0x01,0x02,0x00,0x80,0x04,0x00,0x0b,0x00,0x00,0x00,0x02,0x02,0x00,0x02,0x04,\
            0x00,0x0d,0x00,0x0e,0x00,0x03,0x02,0x00,0x08,0x04,0x00,0x10,\
        },\
    },\
    {0x00,\
        {\
            0x1f,0x06,0x40,0x1c,0x00,0x00,0x00,0x04,0x02,0x00,0x08,0x04,0x00,0x12,0x00,0x00,0x00,0x05,0x02,0x00,\
            0x80,0x04,0x00,0x14,0x00,0x00,0x00,0x06,0x02,0x00,0x0a,0x04,\
        },\
    },\
    {0x00,\
        {\
            0x07,0x06,0x40,0x38,0x00,0x16,0x00,0x17,\
        },\
    },\
    {0x00,\
        {\
            0x13,0x06,0x50,0x00,0x1e,0x94,0x8d,0xf1,0x48,0x31,0x94,0xba,0x75,0x4c,0x3e,0x50,0x00,0x00,0x3d,0x71,\
        },\
    },\
    {0x00,\
        {\
            0x15,0x06,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,\
            0x00,0x00,\
        },\
    },\
    {0x00,\
        {\
            0x06,0x06,0xf0,0x00,0x83,0x6e,0x0c,\
        },\
    },\
}

#endif
